searchNodes=[{"doc":"YubikeyOTP is an Elixir client for validating Yubikey one-time-passwords. It can validate OTPs using Yubico's public API or by using your own or third-party OTP validation services.RequirementsThis module contains all the functions you'll need to authenticate Yubikey OTPs. You will also need a developer API ID (and optionally a shared secret) that can be got from Yubico.You will obviously need at least one Yubikey that supports the OTP protocol.Exampleiex&gt; my_id = Application.get_env(:my_app, :yubikey_client_id) iex&gt; {:ok, service} = YubikeyOTP.service(api_id: my_id) iex&gt; YubikeyOTP.verify(&quot;ccccccclzlojikekndkhfibggvkgujttihkcuvkjfrvj&quot;, service) {:ok, :ok} iex&gt; YubikeyOTP.verify(&quot;ccccccclzlojikekndkhfibggvkgujttihkcuvkjfrvj&quot;, service) {:error, :replayed_otp}","ref":"YubikeyOTP.html","title":"YubikeyOTP","type":"module"},{"doc":"Returns the device ID part of a Yubikey OTPThe first part of a Yubikey OTP is static and identifies the key itself. This ID can be used to match a key with it's owner - you don't want to only authenticate the OTP as valid, you also need to check it's the user's Yubikey.Exampleiex&gt; YubikeyOTP.device_id(&quot;ccccccclulvjtugnjuuufuiebhdvucdihnngndtvfjrb&quot;) {:ok, &quot;ccccccclulvj&quot;}","ref":"YubikeyOTP.html#device_id/1","title":"YubikeyOTP.device_id/1","type":"function"},{"doc":"Returns a Service structure that defines the API backend to use. Default settings are for the Yubicloud service. The only required key is :api_id You don't need to create this for each request - it can be set as a module attribute.Exampleiex&gt; {:ok, service} = YubikeyOTP.service(api_id: &quot;65749337983737&quot;) {:ok, %YubikeyOTP.Service{ api_id: &quot;65749337983737&quot;, api_key: nil, hmac: false, timeout: 1000, timestamp: true, urls: [&quot;https://api.yubico.com/wsapi/2.0/verify&quot;, &quot;https://api2.yubico.com/wsapi/2.0/verify&quot;, &quot;https://api3.yubico.com/wsapi/2.0/verify&quot;, &quot;https://api4.yubico.com/wsapi/2.0/verify&quot;, &quot;https://api5.yubico.com/wsapi/2.0/verify&quot;] }} The Yubicloud API has five different endpoint URLs, and by default these are all used concurrently.","ref":"YubikeyOTP.html#service/1","title":"YubikeyOTP.service/1","type":"function"},{"doc":"Verify a Yubikey OTP using the specified service and return the status This will contact the remote backend and process the response. A successfully authenticated OTP will result in {:ok, :ok}. A failure will result in an :error tuple containing an error code. Most error codes are based on the standard protocol status types and are listed in YubikeyOTP.ErrorsExampleiex&gt; YubikeyOTP.verify(&quot;ccccccclulvjbthgghkbvvlcludiklkncnecncevcrlg&quot;, service) {:ok, :ok} iex&gt; YubikeyOTP.verify(&quot;ccccccclulvjbthgghkbvvlcludiklkncnecncevcrlg&quot;, service) {:error, :replayed_otp}","ref":"YubikeyOTP.html#verify/2","title":"YubikeyOTP.verify/2","type":"function"},{"doc":"Verify a Yubikey OTP using the specified service and return true or false This will contact the remote backend and process the response. An authenticated OTP will produce true. Anything other than a success will be returned as false.Exampleiex&gt; YubikeyOTP.verify?(&quot;ccccccclulvjihcchvujedikcndnbuttfutgvbcgblhk&quot;, service) true iex&gt; YubikeyOTP.verify?(&quot;ccccccclulvjihcchvujedikcndnbuttfutgvbcgblhk&quot;, service) false","ref":"YubikeyOTP.html#verify?/2","title":"YubikeyOTP.verify?/2","type":"function"},{"doc":"Checksums relevant to YubiKeys","ref":"YubikeyOTP.CRC.html","title":"YubikeyOTP.CRC","type":"module"},{"doc":"Computes the CRC16 (ISO 13239) checksum used in the YubiOTP.Examplesiex&gt; YubikeyOTP.CRC.crc16(&quot;&quot;) 65535 iex&gt; YubikeyOTP.CRC.crc16(&lt;&lt;0xffff :: 16&gt;&gt;) 0 iex&gt; YubikeyOTP.CRC.crc16(&quot;1234567890&quot;) 46316","ref":"YubikeyOTP.CRC.html#crc16/2","title":"YubikeyOTP.CRC.crc16/2","type":"function"},{"doc":"Checks whether the given buffer has the correct CRC16 residual.Examples iex&gt; YubikeyOTP.CRC.verify_crc16(&quot;&quot;) false iex&gt; YubikeyOTP.CRC.verify_crc16(&quot;1234567890&quot; &lt;&gt; &lt;&lt;0x13, 0x4b&gt;&gt;) true","ref":"YubikeyOTP.CRC.html#verify_crc16/3","title":"YubikeyOTP.CRC.verify_crc16/3","type":"function"},{"doc":"This module should contain all the errors that can be returned by Yubikey OTP, with descriptions.","ref":"YubikeyOTP.Errors.html","title":"YubikeyOTP.Errors","type":"module"},{"doc":"","ref":"YubikeyOTP.Errors.html#describe/1","title":"YubikeyOTP.Errors.describe/1","type":"function"},{"doc":"","ref":"YubikeyOTP.Errors.html#list/0","title":"YubikeyOTP.Errors.list/0","type":"function"},{"doc":"Implementation of ModHex encoding and decoding.","ref":"YubikeyOTP.ModHex.html","title":"YubikeyOTP.ModHex","type":"module"},{"doc":"Decodes the given modhex into a binary string.An ArgumentError exception is raised if the value is not modhex.Examplesiex&gt; YubikeyOTP.ModHex.decode(&quot;jjnkec&quot;) {:ok, &lt;&lt;0x88, 0xb9, 0x30&gt;&gt;} iex&gt; YubikeyOTP.ModHex.decode(&quot;nope&quot;) :error","ref":"YubikeyOTP.ModHex.html#decode/1","title":"YubikeyOTP.ModHex.decode/1","type":"function"},{"doc":"Decodes the given modhex into a binary string.An ArgumentError exception is raised if the value is not modhex.Examplesiex&gt; YubikeyOTP.ModHex.decode!(&quot;jjnkec&quot;) &lt;&lt;0x88, 0xb9, 0x30&gt;&gt;","ref":"YubikeyOTP.ModHex.html#decode!/1","title":"YubikeyOTP.ModHex.decode!/1","type":"function"},{"doc":"Encodes the binary string to modhex.Examplesiex&gt; YubikeyOTP.ModHex.encode(&lt;&lt;0x88, 0xb9, 0x30&gt;&gt;) &quot;jjnkec&quot;","ref":"YubikeyOTP.ModHex.html#encode/1","title":"YubikeyOTP.ModHex.encode/1","type":"function"},{"doc":"Determines whether the given string is hexadecimal.Examplesiex&gt; YubikeyOTP.ModHex.hex?(&quot;666f6F626172&quot;) true iex&gt; YubikeyOTP.ModHex.hex?(&quot;nope&quot;) false","ref":"YubikeyOTP.ModHex.html#hex?/1","title":"YubikeyOTP.ModHex.hex?/1","type":"function"},{"doc":"Converts the given hexadecimal string to modhex.An ArgumentError exception is raised if the value is not hex.Examplesiex&gt; YubikeyOTP.ModHex.hex_to_modhex!(&quot;88b930&quot;) &quot;jjnkec&quot; iex&gt; YubikeyOTP.ModHex.hex_to_modhex!(&quot;88B930&quot;) &quot;jjnkec&quot;","ref":"YubikeyOTP.ModHex.html#hex_to_modhex!/1","title":"YubikeyOTP.ModHex.hex_to_modhex!/1","type":"function"},{"doc":"Determines whether the given string is modhex.Examplesiex&gt; YubikeyOTP.ModHex.modhex?(&quot;jjnkec&quot;) true iex&gt; YubikeyOTP.ModHex.modhex?(&quot;nope&quot;) false","ref":"YubikeyOTP.ModHex.html#modhex?/1","title":"YubikeyOTP.ModHex.modhex?/1","type":"function"},{"doc":"The digits in modhex","ref":"YubikeyOTP.ModHex.html#modhex_digits/0","title":"YubikeyOTP.ModHex.modhex_digits/0","type":"macro"},{"doc":"Converts the given modhex string to hexadecimal.An ArgumentError exception is raised if the value is not modhex.Examplesiex&gt; YubikeyOTP.ModHex.to_hex!(&quot;jjnkec&quot;) &quot;88b930&quot;","ref":"YubikeyOTP.ModHex.html#to_hex!/1","title":"YubikeyOTP.ModHex.to_hex!/1","type":"function"},{"doc":"OTP FormatThe format of the OTP is documented in:https://developers.yubico.com/OTP/OTPs_Explained.html","ref":"YubikeyOTP.OTP.html","title":"YubikeyOTP.OTP","type":"module"},{"doc":"","ref":"YubikeyOTP.OTP.html#device_id/1","title":"YubikeyOTP.OTP.device_id/1","type":"function"},{"doc":"Parses an OTP into an OTP struct.Without the encryption key, only the public_id, 'prefix,serialandencrypted_otpfields are hydrated. ## Options *:key- provides the 128 bit AES key to decrypt the OTP and load the remaining fields. As part of decryption, the OTP checksum is verified. *:skip_checksum- whether to skip verifying the checksum after decrypting the OTP with the providedkey`. ## Examples iex&gt; YubikeyOTP.OTP.parse(&quot;ccccccclulvjbbhccnndrietjjnkeclcvjgrnhcivtgd&quot;) {:ok, %YubikeyOTP.OTP{ public_id: &quot;ccccccclulvj&quot;, prefix: &quot;cccccc&quot;, serial: 715512, encrypted_otp: &quot;bbhccnndrietjjnkeclcvjgrnhcivtgd&quot; } } iex&gt; YubikeyOTP.OTP.parse(&quot;nope&quot;) :error","ref":"YubikeyOTP.OTP.html#parse/2","title":"YubikeyOTP.OTP.parse/2","type":"function"},{"doc":"Like parse, but returns the OTP struct directly, and throws exceptions when errors are encountered (to permit specific handling, if desired).ExceptionsOTP.ParseError - raised when the OTP cannot be successfully parsed with the given options.OTP.InvalidChecksumError - raised when the checksum of the OTP does not validate.ExamplesWithout specifying a decryption key, only the public information can be hydrated. iex&gt; YubikeyOTP.OTP.parse!(&quot;ccccccclulvjbbhccnndrietjjnkeclcvjgrnhcivtgd&quot;) %YubikeyOTP.OTP{public_id: &quot;ccccccclulvj&quot;, prefix: &quot;cccccc&quot;, serial: 715512, encrypted_otp: &quot;bbhccnndrietjjnkeclcvjgrnhcivtgd&quot; }Specifying a decryption key, but skipping the checksum verification will hydrate the data even with a &quot;bad&quot; decryption. iex&gt; YubikeyOTP.OTP.parse!(&quot;ccccccclulvjbbhccnndrietjjnkeclcvjgrnhcivtgd&quot;, key: &quot;1111111111111111&quot;, skip_checksum: true) %YubikeyOTP.OTP{public_id: &quot;ccccccclulvj&quot;, prefix: &quot;cccccc&quot;, serial: 715512, encrypted_otp: &quot;bbhccnndrietjjnkeclcvjgrnhcivtgd&quot;, private_id: &lt;&lt;68, 48, 254, 248, 123, 61&gt;&gt;, use_counter: 49442, timestamp: 4703963, session_counter: 150, random: &quot;Xn&quot;, checksum: &lt;&lt;1, 15&gt;&gt; } Decrypting the token successfully will hydrate all fields. iex&gt; YubikeyOTP.OTP.parse!(&quot;ccccccclulvjhnblleegivrcjlvvtvujejbclrdjdgvk&quot;, key: &quot;1111111111111111&quot;) %YubikeyOTP.OTP{public_id: &quot;ccccccclulvj&quot;, prefix: &quot;cccccc&quot;, serial: 715512, encrypted_otp: &quot;hnblleegivrcjlvvtvujejbclrdjdgvk&quot;, private_id: &quot;111111&quot;, use_counter: 0, timestamp: 8002816, session_counter: 0, random: &lt;&lt;64, 22&gt;&gt;, checksum: &lt;&lt;44, 51&gt;&gt; } Errors will be thrown when the checksum is invalid, an invalid key is provided, or an invalid token is provided. iex&gt; YubikeyOTP.OTP.parse!(&quot;ccccccclulvjbbhccnndrietjjnkeclcvjgrnhcivtgd&quot;, key: &quot;1111111111111111&quot;) ** (YubikeyOTP.OTP.InvalidChecksumError) OTP checksum is invalid iex&gt; YubikeyOTP.OTP.parse!(&quot;nope&quot;) ** (YubikeyOTP.OTP.ParseError) OTP parsing failed","ref":"YubikeyOTP.OTP.html#parse!/2","title":"YubikeyOTP.OTP.parse!/2","type":"function"},{"doc":"","ref":"YubikeyOTP.OTP.html#validate/2","title":"YubikeyOTP.OTP.validate/2","type":"function"},{"doc":"","ref":"YubikeyOTP.OTP.html#t:t/0","title":"YubikeyOTP.OTP.t/0","type":"type"},{"doc":"Error raised when the OTP checksum is invalid","ref":"YubikeyOTP.OTP.InvalidChecksumError.html","title":"YubikeyOTP.OTP.InvalidChecksumError","type":"exception"},{"doc":"Error raised when parsing the OTP failed","ref":"YubikeyOTP.OTP.ParseError.html","title":"YubikeyOTP.OTP.ParseError","type":"exception"},{"doc":"YubikeyOTPYubikeyOTP is an Elixir client for authenticating Yubikey one-time-passwords. It can verify OTPs using Yubico's public API or by using your own or third-party OTP validation services.In addition to acting as a client, YubikeyOTP's OTP parsing feature can be used to build your own validation service.This early release has not been used in production yet and doesn't have enough tests - please try it and get in touch if something doesn't behave as expected.","ref":"readme.html","title":"YubikeyOTP","type":"extras"},{"doc":"The package can be installed by adding yubikey_otp to your list of dependencies in mix.exs:def deps do [ {:yubikey_otp, &quot;~&gt; 0.2.0&quot;} ] end","ref":"readme.html#installation","title":"YubikeyOTP - Installation","type":"extras"},{"doc":"A Yubikey is a tiny USB device that connects to your computer as if it was USB keyboard. Pressing the button on a Yubikey causes it to type a new unique password that can be checked against a remote server once. Yubikeys can be used for authentication with both command-line and web applications, but are most commonly used alongside traditional passwords to provide Two Factor Authentication on web sites - the key is &quot;something you have&quot;.Most Yubikeys since 2008 can generate OTPs, but modern Yubikeys also support the U2F and WebAuthn standards: these standard are more secure and immune to phishing, and should be prefered for new services. The older OTP format is more widely deployed and still actively used. This library only supports the older OTP format.YubikeyOTP takes the OTP code generated by the Yubikey, sends it to a Yubkey Validation Service to be verified, and parses the API result. It follows Yubico's recommendation to send queries to five different API endpoints simultaneously. my_id = Application.get_env(:my_app, :yubikey_client_id) {:ok, service} = YubikeyOTP.service(api_id: my_id) YubikeyOTP.verify(&quot;ccccccclzlojikekndkhfibggvkgujttihkcuvkjfrvj&quot;, service) # =&gt; {:ok, :ok} YubikeyOTP.verify(&quot;ccccccclzlojikekndkhfibggvkgujttihkcuvkjfrvj&quot;, service) # =&gt; {:error, :replayed_otp}","ref":"readme.html#purpose","title":"YubikeyOTP - Purpose","type":"extras"},{"doc":"You'll need a Yubikey! They can be bought from Yubico or other online stores such as Amazon. The cheaper &quot;Security Key&quot; range does not generate OTPs, only WebAuthn, and won't work with this code.You need to sign up for an API key (using your Yubikey)","ref":"readme.html#requirements","title":"YubikeyOTP - Requirements","type":"extras"},{"doc":"Full API documentation can be found at https://hexdocs.pm/yubikey_otp.","ref":"readme.html#api-documentation","title":"YubikeyOTP - API Documentation","type":"extras"},{"doc":"You can request new features by creating an issue, or submit a pull request with your contribution.","ref":"readme.html#contributing","title":"YubikeyOTP - Contributing","type":"extras"},{"doc":"Thom Dixon","ref":"readme.html#contributors","title":"YubikeyOTP - Contributors","type":"extras"},{"doc":"Copyright (c) 2020 Digital Identity Ltd, UKYubikeyOTP is MIT licensed.","ref":"readme.html#copyright-and-license","title":"YubikeyOTP - Copyright and License","type":"extras"},{"doc":"Wikipedia - YubikeyYubico - Using A Client LibraryYubico - Getting Started Writing ClientsYubico - Validation Protocol V2Yubico - Yubicloud Connector LibrariesYubico - Self Hosted OTP Validation","ref":"readme.html#references","title":"YubikeyOTP - References","type":"extras"},{"doc":"YubikeyOTP is not endorsed by Yubico.","ref":"readme.html#disclaimer","title":"YubikeyOTP - Disclaimer","type":"extras"}]